declare
  rq_id            numeric(16);
  rq_name          varchar(256) := 'Application licensed users';
  rq_desc          varchar(512) := 'Extract the list of applications having the property named ''licencedusers'' set to any value. Lists the attribute values and the active account count per application. This query is part of a broader use case meant to provide visibility on license compliance (unnamed user licence only )';
  rqd              clob;

  counter          numeric(16);
begin
  rqd := 'select distinct a.name as APPLICATION_NAME,
'||'                ap.value as LICENCED_USERS,
'||'                app_lic_users.user_number as USER_NUMBER,
'||'                cast(app_lic_users.user_number * 100 / cast ( ap.value as numeric(31) ) as numeric(16, 0)) || ''%'' as LICENCE_USAGE,
'||'                case
'||'                  when app_lic_users.user_number > cast ( ap.value as numeric(31) ) then
'||'                   ''!''
'||'                  else
'||'                   ''''
'||'                end as LICENSE_STATUS,
'||'                case
'||'                  when cast(app_lic_users.user_number * 100 / cast ( ap.value as numeric(31) ) as numeric(16, 0)) > 120 then
'||'                   ''!''
'||'                  else
'||'                   ''''
'||'                end as LICENCE_TRESHOLD
'||'  from (select user_on_app.application as APPLICATION,
'||'               count(1) as USER_NUMBER
'||'          from (select distinct e.person, a.id as APPLICATION
'||'                  from #pmschema#.employment               e,
'||'                       #pmschema#.entitlement_flat_hier  efh,
'||'                       #pmschema#.application              a,
'||'                       #pmschema#.application_property     ap,
'||'                       #pmschema#.property_key             pk
'||'                 where e.entitlement = efh.parent
'||'                   and efh.child_application = a.id
'||'                   and a.id = ap.application
'||'                   and ap.property_key = pk.id
'||'                   and lower(pk.name) = ''licencedusers''
'||'                   and efh.child_int_type = 1
'||'                   and e.hierarchy = 1) user_on_app
'||'         group by user_on_app.application) app_lic_users,
'||'       #pmschema#.application a,
'||'       #pmschema#.application_property ap,
'||'       #pmschema#.property_key             pk
'||' where app_lic_users.application = a.id
'||'   and a.id = ap.application
'||'   and ap.property_key = pk.id
'||'   and lower(pk.name) = ''licencedusers''
';

  select count(1) into counter from r_query t where lower ( t.name ) = lower ( rq_name );
  if ( counter = 1 ) then
    select t.id into rq_id from r_query t where lower ( t.name ) = lower ( rq_name );

    update r_query_data set query = rqd where r_query = rq_id;
  end if;
  
end;
/

commit;
